@@ -56,6 +56,7 @@ def configure(
         deployment: Optional[str] = None,
         tags: Optional[Dict[str, str]] = None,
         auto_instrument: Optional[bool] = True,
+        upload_on_shutdown: Optional[bool] = True,
         debug_mode: Optional[bool] = False) -> None:
     global _agent
 
@@ -79,6 +80,7 @@ def configure(
         deployment=deployment,
         tags=tags,
         auto_instrument=auto_instrument,
+        upload_on_shutdown=upload_on_shutdown,
         debug_mode=debug_mode)
     _agent.setup()
 
@@ -163,7 +165,8 @@ def upload(block=False) -> None:
 
 def shutdown() -> None:
     global _agent
-    _check_configured()
+    if not _agent:
+        return
 
     atexit.unregister(shutdown)
     _agent.shutdown()
