@@ -25,6 +25,7 @@ class CProfileRecorderTest(unittest.TestCase):
         graphsignal.configure(
             api_key='k1',
             deployment='d1',
+            upload_on_shutdown=False,
             debug_mode=True)
 
     def tearDown(self):
@@ -50,7 +51,6 @@ class CProfileRecorderTest(unittest.TestCase):
         #pp.pprint(MessageToJson(signal))
 
         slow_call = next(call for call in signal.op_profile if 'slow_method' in call.op_name)
-        self.assertEqual(slow_call.profiler_type, signals_pb2.OpStats.ProfilerType.CPROFILE_PROFILER)
         self.assertEqual(slow_call.op_type, signals_pb2.OpStats.OpType.PYTHON_OP)
         self.assertTrue(slow_call.self_host_time_ns > 0)
         self.assertTrue(slow_call.host_time_ns > 0)
