@@ -25,6 +25,7 @@ class KinetoRecorderTest(unittest.TestCase):
         graphsignal.configure(
             api_key='k1',
             deployment='d1',
+            upload_on_shutdown=False,
             debug_mode=True)
 
     def tearDown(self):
@@ -60,7 +61,6 @@ class KinetoRecorderTest(unittest.TestCase):
         self.assertIsNotNone(test_op_stats)
         self.assertTrue(test_op_stats.count >= 1)
         
-        self.assertEqual(test_op_stats.profiler_type, signals_pb2.OpStats.ProfilerType.KINETO_PROFILER)
         self.assertEqual(test_op_stats.op_type, signals_pb2.OpStats.OpType.PYTORCH_OP)
         if torch.cuda.is_available():
             self.assertTrue(test_op_stats.device_time_ns >= 1)
