@@ -24,6 +24,7 @@ class ONNXRuntimeRecorderTest(unittest.TestCase):
         graphsignal.configure(
             api_key='k1',
             deployment='d1',
+            upload_on_shutdown=False,
             debug_mode=True)
 
     def tearDown(self):
@@ -38,6 +39,4 @@ class ONNXRuntimeRecorderTest(unittest.TestCase):
         recorder.on_trace_stop(signal, context, DEFAULT_OPTIONS)
         recorder.on_trace_read(signal, context, DEFAULT_OPTIONS)
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.ONNX_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'ONNX Runtime')
