@@ -7,6 +7,7 @@ import time
 from unittest.mock import patch, Mock
 from google.protobuf.json_format import MessageToJson
 import pprint
+import types
 import openai
 
 import graphsignal
@@ -25,6 +26,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         graphsignal.configure(
             api_key='k1',
             deployment='d1',
+            upload_on_shutdown=False,
             debug_mode=True)
 
     async def asyncTearDown(self):
@@ -39,9 +41,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         recorder.on_trace_stop(signal, context, DEFAULT_OPTIONS)
         recorder.on_trace_read(signal, context, DEFAULT_OPTIONS)
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.OPENAI_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
 
     @patch.object(Uploader, 'upload_signal')
     @patch.object(openai.Completion, 'create')
@@ -93,9 +93,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         #pp = pprint.PrettyPrinter()
         #pp.pprint(MessageToJson(signal))
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.OPENAI_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
 
         self.assertEqual(find_param(signal, 'model'), 'text-davinci-003')
         self.assertEqual(find_param(signal, 'max_tokens'), '1024')
@@ -113,6 +111,74 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         self.assertEqual(find_data_metric(signal, 'completion', 'finish_reason_stop'), 1.0)
         self.assertEqual(find_data_metric(signal, 'completion', 'finish_reason_length'), 1.0)
 
+    @patch.object(Uploader, 'upload_signal')
+    @patch.object(openai.Completion, 'create')
+    async def test_completion_create_stream(self, mocked_create, mocked_upload_signal):
+        # mocking overrides autoinstrumentation, reinstrument
+        recorder = OpenAIRecorder()
+        recorder.setup()
+        recorder._is_sampling = True
+
+        test_ret = [
+            {
+                "choices": [
+                    {
+                    "finish_reason": None,
+                    "index": 1,
+                    "logprobs": None,
+                    "text": "\n"
+                    }
+                ],
+                "created": 1676896808,
+                "id": "cmpl-6lzkernKqOvF4ewZGhHlZ63HZJcbc",
+                "model": "text-davinci-003",
+                "object": "text_completion"
+            },
+            {
+                "choices": [
+                    {
+                    "finish_reason": None,
+                    "index": 1,
+                    "logprobs": None,
+                    "text": "\n"
+                    }
+                ],
+                "created": 1676896808,
+                "id": "cmpl-6lzkernKqOvF4ewZGhHlZ63HZJcbc",
+                "model": "text-davinci-003",
+                "object": "text_completion"
+            }
+        ]
+        def test_ret_gen():
+            for item in test_ret:
+                yield item
+        mocked_create.return_value = test_ret_gen()
+
+        response = openai.Completion.create(
+            model="text-davinci-003", 
+            prompt=['count 1 to 3', 'generate 2 random letters'],
+            temperature=0.1,
+            top_p=1,
+            max_tokens=1024,
+            frequency_penalty=0,
+            presence_penalty=0,
+            stream=True)
+
+        for r in response:
+            pass
+
+        recorder.shutdown()
+
+        signal = mocked_upload_signal.call_args[0][0]
+
+        #pp = pprint.PrettyPrinter()
+        #pp.pprint(MessageToJson(signal))
+
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
+        self.assertEqual(signal.root_span.spans[0].name, 'response')
+        self.assertTrue(signal.root_span.spans[0].start_ns > 0)
+        self.assertTrue(signal.root_span.spans[0].end_ns > 0)
+
     @patch.object(Uploader, 'upload_signal')
     @patch.object(openai.Completion, 'acreate')
     async def test_completion_acreate(self, mocked_create, mocked_upload_signal):
@@ -163,9 +229,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         #pp = pprint.PrettyPrinter()
         #pp.pprint(MessageToJson(signal))
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.OPENAI_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
 
         self.assertEqual(find_param(signal, 'model'), 'text-davinci-003')
         self.assertEqual(find_param(signal, 'max_tokens'), '1024')
@@ -181,6 +245,81 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         self.assertEqual(find_data_metric(signal, 'completion', 'element_count'), 2.0)
         self.assertEqual(find_data_metric(signal, 'completion', 'token_count'), 96.0)
 
+
+    @patch.object(Uploader, 'upload_signal')
+    @patch.object(openai.ChatCompletion, 'create')
+    async def test_chat_completion_create(self, mocked_create, mocked_upload_signal):
+        # mocking overrides autoinstrumentation, reinstrument
+        recorder = OpenAIRecorder()
+        recorder.setup()
+        recorder._is_sampling = True
+
+        mocked_create.return_value = {
+            "choices": [
+                {
+                    "finish_reason": "stop",
+                    "index": 0,
+                    "logprobs": None,
+                    "message": {
+                        "content": "\n\ntest completion 1"
+                    }
+                },
+                {
+                    "finish_reason": "length",
+                    "index": 1,
+                    "logprobs": None,
+                    "message": {
+                        "content": "\n\ntest completion 222"
+                    }
+                }
+            ],
+            "created": 1675070122,
+            "id": "cmpl-id",
+            "model": "gpt-3.5-turbo",
+            "object": "text_completion",
+            "usage": {
+                "completion_tokens": 96,
+                "prompt_tokens": 6,
+                "total_tokens": 102
+            }
+        }
+
+        response = openai.ChatCompletion.create(
+            model="gpt-3.5-turbo", 
+            messages=[
+                {'role': 'system', 'content': 'test prompt 1'}, 
+                {'role': 'system', 'content': 'test prompt 2'}],
+            temperature=0.1,
+            top_p=1,
+            max_tokens=1024,
+            frequency_penalty=0,
+            presence_penalty=0)
+
+        recorder.shutdown()
+
+        signal = mocked_upload_signal.call_args[0][0]
+
+        #pp = pprint.PrettyPrinter()
+        #pp.pprint(MessageToJson(signal))
+
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
+
+        self.assertEqual(find_param(signal, 'model'), 'gpt-3.5-turbo')
+        self.assertEqual(find_param(signal, 'max_tokens'), '1024')
+        self.assertEqual(find_param(signal, 'temperature'), '0.1')
+        self.assertEqual(find_param(signal, 'top_p'), '1')
+        self.assertEqual(find_param(signal, 'presence_penalty'), '0')
+        self.assertEqual(find_param(signal, 'frequency_penalty'), '0')
+
+        self.assertEqual(find_data_metric(signal, 'messages', 'byte_count'), 26.0)
+        self.assertEqual(find_data_metric(signal, 'messages', 'element_count'), 2.0)
+        self.assertEqual(find_data_metric(signal, 'messages', 'token_count'), 6.0)
+        self.assertEqual(find_data_metric(signal, 'completion', 'byte_count'), 40.0)
+        self.assertEqual(find_data_metric(signal, 'completion', 'element_count'), 2.0)
+        self.assertEqual(find_data_metric(signal, 'completion', 'token_count'), 96.0)
+        self.assertEqual(find_data_metric(signal, 'completion', 'finish_reason_stop'), 1.0)
+        self.assertEqual(find_data_metric(signal, 'completion', 'finish_reason_length'), 1.0)
+
     @patch.object(Uploader, 'upload_signal')
     @patch.object(openai.Edit, 'create')
     async def test_edits_create(self, mocked_create, mocked_upload_signal):
@@ -219,9 +358,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         #pp = pprint.PrettyPrinter()
         #pp.pprint(MessageToJson(signal))
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.OPENAI_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
 
         self.assertEqual(find_param(signal, 'model'), 'text-davinci-edit-001')
         self.assertEqual(find_param(signal, 'temperature'), '0.1')
@@ -284,9 +421,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         #pp = pprint.PrettyPrinter()
         #pp.pprint(MessageToJson(signal))
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.OPENAI_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
 
         self.assertEqual(find_param(signal, 'engine'), 'text-embedding-ada-002')
 
@@ -326,9 +461,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         #pp = pprint.PrettyPrinter()
         #pp.pprint(MessageToJson(signal))
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.OPENAI_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
 
         self.assertEqual(find_param(signal, 'n'), '1')
         self.assertEqual(find_param(signal, 'size'), '256x256')
@@ -384,9 +517,7 @@ class OpenAIRecorderTest(unittest.IsolatedAsyncioTestCase):
         #pp = pprint.PrettyPrinter()
         #pp.pprint(MessageToJson(signal))
 
-        self.assertEqual(
-            signal.frameworks[0].type,
-            signals_pb2.FrameworkInfo.FrameworkType.OPENAI_FRAMEWORK)
+        self.assertEqual(signal.frameworks[0].name, 'OpenAI Python Library')
 
         self.assertEqual(find_param(signal, 'model'), 'text-moderation-latest')
 
